<?php

namespace app\controllers;

use Yii;
use app\models\Acao;
use app\models\Delegados;
use app\models\DelegadosSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * DelegadosController implements the CRUD actions for Delegados model.
 */
class DelegadosController extends Controller
{
    /**
     * @inheritDoc
     */
    public function behaviors()
    {
        return array_merge(
            parent::behaviors(),
            [
                'verbs' => [
                    'class' => VerbFilter::className(),
                    'actions' => [
                        'delete' => ['POST'],
                    ],
                ],
            ]
        );
    }

    /**
     * Lists all Delegados models.
     *
     * @return string
     */
    public function actionIndex()
    {
        $searchModel = new DelegadosSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams); 
        $dataProvider->pagination->pageSize = 10;
      if (isset($_GET['id_opracao'])) {
            $idOperacao = $_GET['id_opracao'];
            $acoes = \Yii::$app->AdministracaoComponent->getAtionMenu($idOperacao);
            $acaoCriar = \Yii::$app->AdministracaoComponent->getAcaoCria();
            Yii::$app->session['id_op'] = $idOperacao;
        } else if (Yii::$app->session['id_op']) {
            $idOperacao = Yii::$app->session['id_op'];
            $acoes = \Yii::$app->AdministracaoComponent->getAtionMenu($idOperacao);
            $acaoCriar = \Yii::$app->AdministracaoComponent->getAcaoCria();
        }
        $model = \app\models\Menus::find()->where(['ESTADO'=>'A','ID'=>$idOperacao])->one();
        Yii::$app->session['rootDesigAdmin'] = $model->sELF->DS_MENU;

        return $this->render('index', [
            'acoes' => $acoes,
            'acaoCriar' => $acaoCriar,
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'idOperacao' => $model->DS_MENU,
            'rootDesig' => Yii::$app->session['rootDesigAdmin'],
        ]);






    }

    /**
     * Displays a single Delegados model.
     * @param string $id ID
     * @return string
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Delegados model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return string|\yii\web\Response
     */
    public function actionCreate()
    {
        $model = new Delegados();

        if ($this->request->isPost) {
            if ($model->load($this->request->post()) && $model->save()) {
                return $this->redirect(['view', 'id' => $model->id]);
            }
        } else {
            $model->loadDefaultValues();
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Delegados model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param string $id ID
     * @return string|\yii\web\Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($this->request->isPost && $model->load($this->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Delegados model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param string $id ID
     * @return \yii\web\Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Delegados model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param string $id ID
     * @return Delegados the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Delegados::findOne(['id' => $id])) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
