<?php

namespace app\controllers;

use Yii;
use app\models\Menus;
use app\models\MenusSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * MenusController implements the CRUD actions for Menus model.
 */
class MenusController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Menus models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new MenusSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        $dataProvider->pagination->pageSize = 10;
        if (isset($_GET['id_opracao'])) {
            $idOperacao = $_GET['id_opracao'];
            $acoes = \Yii::$app->AdministracaoComponent->getAtionMenu($idOperacao);
            $acaoCriar = \Yii::$app->AdministracaoComponent->getAcaoCria();
            Yii::$app->session['id_op'] = $idOperacao;
        } else if (Yii::$app->session['id_op']) {
            $idOperacao = Yii::$app->session['id_op'];
            $acoes = \Yii::$app->AdministracaoComponent->getAtionMenu($idOperacao);
            $acaoCriar = \Yii::$app->AdministracaoComponent->getAcaoCria();
        }
        $model = \app\models\Menus::find()->where(['ID'=>$idOperacao,'ESTADO'=>'A'])->one();
        Yii::$app->session['rootDesigAdmin'] = $model->sELF->DS_MENU;
        return $this->render('index', [
            'acoes' => $acoes,
            'acaoCriar' => $acaoCriar,
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'idOperacao' => $model->DS_MENU,
            'rootDesig' => Yii::$app->session['rootDesigAdmin'],
        ]);
    }

    /**
     * Displays a single Menus model.
     * @param string $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->renderAjax('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Menus model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Menus();
        $model->ESTADO = "A";
        $model->ID = \Faker\Provider\Uuid::uuid();
        
        if ($model->load(Yii::$app->request->post())){
            if($model->SELF_ID == ''){
                //quando cria um menu principal
                $menu = Menus::find()->where(['ESTADO'=>'A','SELF_ID'=>NULL])->orderBy(['ORDEM' => SORT_DESC])->one();
                $menu? $resultado = $menu->ORDEM: $resultado = 0;
                //quando atualiza um menu principal 
                if($model->ORDEM !== ''){
                    $flag = \Yii::$app->db->createCommand("UPDATE menus SET ORDEM=:ordem_new WHERE SELF_ID=:self and ORDEM=:order_old")
                            ->bindValue(':ordem_new', ($resultado+1))
                            ->bindValue(':self', NULL)
                            ->bindValue(':order_old', $model->ORDEM)
                            ->execute();   
                }else{
                    $model->ORDEM = ($resultado+1);
                }
                $model->SELF_ID = NULL;
            }
            if($model->save()) {
                return $this->redirect(['index']);
            }
        } else {
            return $this->renderAjax('create', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Updates an existing Menus model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param string $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post())){
            $model->SELF_ID = $model->SELF_ID == ''? NULL : $model->SELF_ID;
            if($model->save()) {
               return $this->redirect(['index']); 
            }
        } else {
            return $this->renderAjax('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing Menus model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param string $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $model = $this->findModel($id);
        $model->ESTADO = 'I';
        $model->update();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Menus model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param string $id
     * @return Menus the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Menus::find()->where(['ESTADO'=>'A','ID'=>$id])->one()) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
}
