<?php

namespace app\controllers;

use Yii;
use app\models\Perfil;
use app\models\PerfilSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * PerfilController implements the CRUD actions for Perfil model.
 */
class PerfilController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Perfil models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new PerfilSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        if (isset($_GET['id_opracao'])) {
            $idOperacao = $_GET['id_opracao'];
            $acoes = \Yii::$app->AdministracaoComponent->getAtionMenu($idOperacao);
            $acaoCriar = \Yii::$app->AdministracaoComponent->getAcaoCria();
            Yii::$app->session['id_op'] = $idOperacao;
        } else if (Yii::$app->session['id_op']) {
            $idOperacao = Yii::$app->session['id_op'];
            $acoes = \Yii::$app->AdministracaoComponent->getAtionMenu($idOperacao);
            $acaoCriar = \Yii::$app->AdministracaoComponent->getAcaoCria();
        }
        $model = \app\models\Menus::find()->where(['ID'=>$idOperacao,'ESTADO'=>'A'])->one();
        Yii::$app->session['rootDesigAdmin'] = $model->sELF->DS_MENU;
        return $this->render('index', [
            'acoes' => $acoes,
            'acaoCriar' => $acaoCriar,
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'idOperacao' => $model->DS_MENU,
            'rootDesig' => Yii::$app->session['rootDesigAdmin'],
        ]);
    }

    /**
     * Displays a single Perfil model.
     * @param string $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->renderAjax('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Perfil model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Perfil();
        
        $model->ESTADO = "A";
        $model->ID = \Faker\Provider\Uuid::uuid();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['index']);
        } else {
            return $this->renderAjax('create', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Updates an existing Perfil model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param string $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['index']);
        } else {
            return $this->renderAjax('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing Perfil model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param string $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $model = $this->findModel($id);
        $model->ESTADO = 'I';
        $model->update();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Perfil model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param string $id
     * @return Perfil the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Perfil::find()->where(['ESTADO'=>'A','ID'=>$id])->one()) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
}
