<?php

namespace app\controllers;

use Yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\filters\VerbFilter;
use app\models\LoginForm;
use app\models\ContactForm;

class SiteController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'only' => ['index'],
                'rules' => [
                    [
                        //'actions' => ['logout'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'logout' => ['post'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    public function actionIndex()
    {
        return $this->render('index');
    }

    public function actionLogin()
    {
        if (!Yii::$app->user->isGuest) {
            return $this->goHome();
        }

        $model = new LoginForm();
        if ($model->load(Yii::$app->request->post()) && $model->login()) {
            return $this->goBack();
        }
        return $this->render('login', [
            'model' => $model,
        ]);
    }

    public function actionLogout()
    {
        Yii::$app->user->logout();

        return $this->redirect(['login']);
    }

    public function actionContact()
    {
        $model = new ContactForm();
        if ($model->load(Yii::$app->request->post()) && $model->contact(Yii::$app->params['adminEmail'])) {
            Yii::$app->session->setFlash('contactFormSubmitted');

            return $this->refresh();
        }
        return $this->render('contact', [
            'model' => $model,
        ]);
    }

    public function actionAbout()
    {
        return $this->render('about');
    }
    
    public function actionVerMenusPerfil($perfil_id)
	{
      return Yii::$app->AdministracaoComponent->DesenharPerfilMenuTree($perfil_id);
    }
    
	public function actionCriarEliminarPerfilmenu($acao,$perfil_id,$menu_id) 
	{ 
      return Yii::$app->AdministracaoComponent->CriarEliminarPermisao($acao,$perfil_id,$menu_id);
    }
    
	public function actionListmenus($id) 
	{
       return Yii::$app->AdministracaoComponent->desenharMenus($id);
    }
    
    public function actionPerfil()
    {
        return $this->render('perfil');
    }
    
    public function actionLockScreen()
    {
        if(isset(Yii::$app->user->identity->USERNAME)){
            // save current username    
            $username = Yii::$app->user->identity->USERNAME;

            // force logout     
            Yii::$app->user->logout();

            // render form lockscreen
            $model = new LoginForm(); 
            $model->username = $username;    //set default value 
            return $this->render('lockscreen', [
                'model' => $model,
            ]);
        }else{
            return $this->redirect(['login']);
        }
    }
    
    public function actionAlterarpassw($id) 
	{
        $db_pass = Yii::$app->user->identity->PASSW;
        $model = \app\models\User::find()->where(['ESTADO'=>'A','ID'=>$id])->one(); 
        $pass_user_old = $model->PASSW;
        if ($model->load(Yii::$app->request->post())) {  
            if($model->save(FALSE)){; 
                return $this->redirect('index.php?r=site/perfil');
            }
        } else {
            $model->PASSW = '';
            $model->PASSWCONFIRMA = '';
            return $this->renderAjax('muda_passw', [
                'model' => $model,
                'password' => $db_pass,
            ]);
        }
    }
    
    public function actionGrafico()
	{
        $procedimento = \app\models\RelProcedimentoDente::find()->all();
        $proced = [];
        if($procedimento){
            foreach ($procedimento as $proc){
                $proced['data'] = $proc->DATA;
                $proced['procedimento'] = $proc->pROCEDIMENTO->NOME;
            }
        }
        return json_encode($proced);
    }
}
