<?php

namespace app\controllers;

use Yii;
use app\models\EventosTiket;
use app\models\EventosLocal;
use app\models\TiketRegistros;
use app\models\TiketRegistrosSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use kartik\mpdf\Pdf;
 require('../web/phpqrcode/qrlib.php');
//use Da\QrCode\QrCode;
/**
 * TiketRegistrosController implements the CRUD actions for TiketRegistros model.
 */
class TiketRegistrosController extends Controller
{
    /**
     * @inheritDoc
     */
    public function behaviors()
    {
        return array_merge(
            parent::behaviors(),
            [
                'verbs' => [
                    'class' => VerbFilter::className(),
                    'actions' => [
                        'delete' => ['POST'],
                    ],
                ],
            ]
        );
    }

    /**
     * Lists all TiketRegistros models.
     *
     * @return string
     */
    public function actionIndex()
    {
        $searchModel = new TiketRegistrosSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
 
        $dataProvider->pagination->pageSize = 10;
      if (isset($_GET['id_opracao'])) {
            $idOperacao = $_GET['id_opracao'];
            $acoes = \Yii::$app->AdministracaoComponent->getAtionMenu($idOperacao);
            $acaoCriar = \Yii::$app->AdministracaoComponent->getAcaoCria();
            Yii::$app->session['id_op'] = $idOperacao;
        } else if (Yii::$app->session['id_op']) {
            $idOperacao = Yii::$app->session['id_op'];
            $acoes = \Yii::$app->AdministracaoComponent->getAtionMenu($idOperacao);
            $acaoCriar = \Yii::$app->AdministracaoComponent->getAcaoCria();
        }
        $model = \app\models\Menus::find()->where(['ESTADO'=>'A','ID'=>$idOperacao])->one();
        Yii::$app->session['rootDesigAdmin'] = $model->sELF->DS_MENU; 

        return $this->render('index', [
            'acoes' => $acoes,
            'acaoCriar' => $acaoCriar,
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'idOperacao' => $model->DS_MENU,
            'rootDesig' => Yii::$app->session['rootDesigAdmin'],
        ]);

       
    }

    /**
     * Displays a single TiketRegistros model.
     * @param string $id ID
     * @return string
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
       

        $model = $this->findModel($id);
         $qrCodeName = "imagem_qrcode_{$model->id}.png"; 

         \QRcode::png("Id:$model->id Name:$model->nome Categoria:$model->categoria", $qrCodeName );

          
        return $this->render('view', [
            'model' => $model,
            'qrCodeName' => $qrCodeName,
        ]);
    }

    /**
     * Creates a new TiketRegistros model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return string|\yii\web\Response
     */
    public function actionCreate()
    {
        $model = new TiketRegistros(); 
        
        $model->estado = "7";
        $model->id = \Faker\Provider\Uuid::uuid();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['index']);
        } else {
            return $this->renderAjax('create', [
                'model' => $model,
            ]);
        }



    }

    /**
     * Updates an existing TiketRegistros model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param string $id ID
     * @return string|\yii\web\Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save(false)) {
            return $this->redirect(['index']);
        } else {
            return $this->renderAjax('update', [
                'model' => $model,
            ]);
        }
    }   

    public function actionImprimirTicket($id)
    {
        $qrcodeUlr = "";
 
       $model = $this->findModel($id);

       $string = $model->id."\n".$model->nome." ".$model->apelido;

       //$qrcodeUlr = "https://chart.googleapis.com/chart?chs=200x200&cht=qr&chl=$string&choe=UTF-8";

       if(!empty($model->qrcode)){
             $qrcodeUlr = $model->qrcode; 
       } else {
        $qrcodeUlr = "images/qrcode_{$model->id}.png"; 
        \QRcode::png("$model->id$model->local $model->nome $model->apelido $model->categoria", $qrcodeUlr );
        $model->qrcod = $qrcodeUlr;
        $model->save(FALSE);
       }



        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['index']);
        } else {
            return $this->renderAjax('vticket', [
                'model' => $model,
                'qrcodeUlr' => $qrcodeUlr,
                //'qrCodeName' => $qrCodeName
            ]);
        }
    }

   
    public function actionReport($id) {

       $model = $this->findModel($id);

       $string = $model->id."\n".$model->nome." ".$model->apelido;

       $qrcodeUlr = "https://chart.googleapis.com/chart?chs=200x200&cht=qr&chl=$string&choe=UTF-8";


    // get your HTML raw content without any layouts or scripts
    $content = $this->renderPartial('vticket', [
                'model' => $model,
                'qrcodeUlr' => $qrcodeUlr
            ]);
    
    // setup kartik\mpdf\Pdf component
    $pdf = new Pdf([
        // set to use core fonts only
        'mode' => Pdf::MODE_CORE, 
        // A4 paper format
        'format' => Pdf::FORMAT_A4, 
        // portrait orientation
        'orientation' => Pdf::ORIENT_PORTRAIT, 
        // stream to browser inline
        'destination' => Pdf::DEST_BROWSER, 
        // your html content input
        'content' => $content,  
        // format content from your own css file if needed or use the
        // enhanced bootstrap css built by Krajee for mPDF formatting 
        'cssFile' => '@vendor/kartik-v/yii2-mpdf/src/assets/kv-mpdf-bootstrap.min.css',
        // any css to be embedded if required
        'cssInline' => '.kv-heading-1{font-size:18px}', 
         // set mPDF properties on the fly
        'options' => ['title' => 'Krajee Report Title', 'ID'=>""],
         // call mPDF methods on the fly
        'methods' => [ 
            'SetHeader'=>['Krajee Report Header'], 
            'SetFooter'=>['{PAGENO}'],
        ]
    ]);
    
    // return the pdf output as per the destination setting
    return $pdf->render(); 
}

public function actionImport()
{
 
 /*   $row = 1;
    if (($handle = fopen("../web/file/test.csv", "r")) !== FALSE) {
        while (($data = fgetcsv($handle, 1000, ";")) !== FALSE) {

             $model = new TiketRegistros();         
            $model->estado = "7";
            $model->id = \Faker\Provider\Uuid::uuid();
            $model->categoria =  "International Artists";
            //$model->data_registo = 
            $num = count($data);
            echo "<p> $num fields in line $row: <br /></p>\n";
            
            if($row <> 1){
                for ($c=0; $c < $num; $c++) {
                    //LAST NAME   FIRT NAME   COUNTRY COMPANY
                    $model->apelido =  utf8_encode($data[0]);
                    $model->nome =  utf8_encode($data[1]);
                    $model->pais =  utf8_encode($data[2]);
                    $model->solicta_destino =  utf8_encode($data[3]);
                   
 
//id qrcod  nome  apelido pais respondido   solicta_destino local_partida local_chegada numero_doc propal categoria local token estado data_registo
                    echo utf8_encode($data[$c]) . "\n";
                }
            }
             $model->save(FALSE);
            $row++;
        }
        fclose($handle);
    }
 */


}



public function actionVerifyQrcod()
{
    \Yii::$app->response->format = \yii\web\Response:: FORMAT_JSON;
    //$attributes = \yii::$app->request->get();

    if($_GET){
         
           $string =  substr($_GET["freepass"], 0, 36);  
            $id = $string;         
           
         $tiketRegistros = TiketRegistros::find()->where(['id' => $id])->one();         
          
          if($tiketRegistros )
          {

            $modeleventoticket= EventosTiket::find()->where(['id_registo' => $id ])->one();  //$this->findEventosTiket($id);

           if(!$modeleventoticket){
                $modeleventoticket = new EventosTiket();
                $modeleventoticket->id = \Faker\Provider\Uuid::uuid();
                $modeleventoticket->id_registo = $id;
                $modeleventoticket->qrcode = $tiketRegistros->qrcod;
                $modeleventoticket->entra_saida = "in";
                $modeleventoticket->estado = "7";
                $modeleventoticket->evento = $tiketRegistros->local;
                //$modeleventoticket->data_registo =     
                $modeleventoticket->save(false);     
           } else {


           }

           return array('status' => true, 'data'=> "success! $tiketRegistros->nome $tiketRegistros->apelido", 'total_entrado'=> \Yii::$app->ConsultasComponent->CountEntradasComponent($tiketRegistros->local));
          
          }
        else
        {
           return array('status'=>false,'data'=> "No Ticket Found.  $id ");
        } 
    }
    else
    {
       return array('status'=>false,'data'=> 'No Ticket Found2');
    }  

}

public function actionDashboardApi()
{
    \Yii::$app->response->format = \yii\web\Response:: FORMAT_JSON;
    //$attributes = \yii::$app->request->get();

    if(isset($_GET["local"])){            
                   
        if($modelLocal = EventosLocal::find()->where(['id' => $_GET["local"] ])->one() ){

            return array('status' => true, 'data'=> "success!", 
                    'evento'=>$modelLocal->evento,
                    'total_entrado'=> \Yii::$app->ConsultasComponent->CountEntradasComponent($_GET["local"]),
                    'Delegado'=> \Yii::$app->ConsultasComponent->CountContDelegatesComponent($_GET["local"], 'Delegado'),
                    'International Artists'=> \Yii::$app->ConsultasComponent->CountContDelegatesComponent($_GET["local"], "International Artists"),
                    'Staff'=> \Yii::$app->ConsultasComponent->CountContDelegatesComponent($_GET["local"], "Staff"),
                    'Jornalista'=> \Yii::$app->ConsultasComponent->CountContDelegatesComponent($_GET["local"], "Jornalista"),
            );
        } else {

            return array('status'=>false,'data'=> 'No results found');
        }
          
    } 
    else
    {
       return array('status'=>false,'data'=> 'No results found');
    }  
        
}



public function actionDelete($id)
{
    $this->findModel($id)->delete();

    return $this->redirect(['index']);
}

/**
 * Finds the TiketRegistros model based on its primary key value.
 * If the model is not found, a 404 HTTP exception will be thrown.
 * @param string $id ID
 * @return TiketRegistros the loaded model
 * @throws NotFoundHttpException if the model cannot be found
 */
protected function findModel($id)
{
    if (($model = TiketRegistros::findOne(['id' => $id])) !== null) {
        return $model;
    }

    throw new NotFoundHttpException('The requested page does not exist.');
}

protected function findEventosTiket($id)
{
    if (($model = EventosTiket::findOne(['id' => $id])) !== null) {
        return $model;
    }

    throw new NotFoundHttpException('The requested page does not exist.');
}



}